import { Host, h, } from "@stencil/core";
import { onComponentRequiredPropUndefined, addFormResetListener, removeFormResetListener, removeDisabledFalse, isPropDefined, isSlotUsed, slotHasContent, } from "../../utils/helpers";
const ADDITIONAL_FIELD = "additional-field";
/**
 * @slot additional-field - Content to displayed alongside a radio option.
 */
export class RadioOption {
    constructor() {
        this.defaultRadioValue = "";
        this.hasAdditionalField = false;
        this.skipFocus = false;
        /**
         * The style of additionalField that will be displayed if used.
         */
        this.additionalFieldDisplay = "static";
        /**
         * If `true`, the disabled state will be set.
         */
        this.disabled = false;
        /**
         * The text to be displayed when dynamic.
         */
        this.dynamicText = "This selection requires additional answers";
        /**
         * Sets the theme color to the dark or light theme color. "inherit" will set the color based on the system settings or ic-theme component.
         */
        this.theme = "inherit";
        /**
         * If `true`, the radio option will be displayed in a selected state.
         */
        this.selected = false;
        this.initiallySelected = this.selected;
        this.handleClick = (event) => {
            var _a, _b;
            const clickedAdditionalField = event.target.matches(".dynamic-container:not(ic-radio-option, .checkmark), .dynamic-container *:not(ic-radio-option, .checkmark)");
            if (!this.disabled &&
                (!clickedAdditionalField || event.target === this.radioElement)) {
                event.stopPropagation();
                if (this.skipFocus === false) {
                    (_a = this.radioElement) === null || _a === void 0 ? void 0 : _a.focus();
                }
                this.skipFocus = false;
                if (this.hasAdditionalField) {
                    this.value = ((_b = this.getAdditionalField()) === null || _b === void 0 ? void 0 : _b.value) || this.defaultRadioValue;
                }
                this.icCheck.emit({
                    value: this.value,
                });
            }
        };
        this.handleKeyDown = (event) => {
            const preventPropagationElements = ["IC-DATE-INPUT", "IC-DATE-PICKER"];
            if (this.getAdditionalField() == document.activeElement &&
                preventPropagationElements.includes(this.getAdditionalField().nodeName)) {
                event.stopPropagation();
            }
        };
        this.handleFormReset = () => {
            this.skipFocus = true;
            this.selected = this.initiallySelected;
        };
    }
    watchDisabledHandler() {
        removeDisabledFalse(this.disabled, this.el);
    }
    watchSelectedHandler() {
        this.icSelectedChange.emit();
    }
    handleCheck(ev) {
        if (this.additionalFieldDisplay === "static" &&
            Array.from(this.el.querySelectorAll("ic-radio-option")).includes(ev.target))
            this.icCheck.emit({ value: this.value });
    }
    disconnectedCallback() {
        removeFormResetListener(this.el, this.handleFormReset);
    }
    componentWillLoad() {
        if (isSlotUsed(this.el, ADDITIONAL_FIELD)) {
            this.hasAdditionalField = true;
            this.getAdditionalField();
        }
        this.defaultRadioValue = this.value;
        addFormResetListener(this.el, this.handleFormReset);
        removeDisabledFalse(this.disabled, this.el);
    }
    componentDidLoad() {
        onComponentRequiredPropUndefined([{ prop: this.value, propName: "value" }], "Radio Option");
    }
    componentWillRender() {
        const hasSlot = slotHasContent(this.el, ADDITIONAL_FIELD);
        if (hasSlot && !this.hasAdditionalField) {
            this.hasAdditionalField = true;
            this.getAdditionalField();
        }
        else if (!hasSlot && this.hasAdditionalField) {
            this.hasAdditionalField = false;
        }
    }
    componentDidRender() {
        if (this.additionalFieldDisplay === "static") {
            const additionalField = this.getAdditionalField();
            if (!this.selected || this.disabled) {
                additionalField === null || additionalField === void 0 ? void 0 : additionalField.setAttribute("disabled", "");
            }
            else {
                additionalField === null || additionalField === void 0 ? void 0 : additionalField.removeAttribute("disabled");
            }
        }
    }
    additionalFieldValueHandler(event) {
        if (this.selected) {
            this.value = event.detail.value || this.defaultRadioValue;
            this.icCheck.emit({
                value: this.value,
            });
        }
        event.stopImmediatePropagation();
    }
    /**
     * Sets focus on the radio option.
     */
    async setFocus() {
        var _a;
        (_a = this.radioElement) === null || _a === void 0 ? void 0 : _a.focus();
    }
    /**
     * @internal Sets the tabIndex of the radio option.
     */
    async setTabIndex(value) {
        var _a;
        (_a = this.radioElement) === null || _a === void 0 ? void 0 : _a.setAttribute("tabIndex", value.toString());
    }
    getAdditionalField() {
        const additionalField = this.el.querySelector('ic-text-field[slot="additional-field"]');
        if (additionalField)
            additionalField.hiddenInput = false;
        return additionalField;
    }
    render() {
        const { additionalFieldDisplay, disabled, dynamicText, form, groupLabel, handleClick, handleKeyDown, hasAdditionalField, label, name, selected, value, theme, } = this;
        const id = `ic-radio-option-${isPropDefined(label) || value}-${groupLabel}`;
        return (h(Host, { key: 'e775db934af98431874bc7b25a931de822c1a987', onClick: handleClick, onKeyDown: handleKeyDown, class: {
                ["ic-radio-option-disabled"]: !!disabled,
                [`ic-theme-${theme}`]: theme !== "inherit",
            } }, h("div", { key: 'e80a4acc065eb631c897d3c35b03f1747f272acb', class: { container: true, disabled: !!disabled } }, h("div", { key: 'b5d2a7fb6447ce7e365f8661e002767a8f909280' }, h("input", { key: 'd352f22e5f86533b4bdfe6e23be01993ef5dc2a9', tabindex: selected ? "0" : "-1", type: "radio", name: name, id: id, value: value, disabled: disabled, checked: selected, ref: (el) => (this.radioElement = el), form: form }), h("span", { key: '1718b2c24e76c82f98093b78bac5917da201af4d', class: "checkmark" })), h("ic-typography", { key: '3938c3a33f1534fa8778abed7e09cbc496af0612', class: "radio-label", variant: "body" }, h("label", { key: '586877cf41eafe536366cdd949d2330aa82aaaed', htmlFor: id }, label))), hasAdditionalField && (h("div", { key: '3f7d2a81b51a533843b74b59a730ad3231752638', class: {
                "dynamic-container": true,
                hidden: additionalFieldDisplay === "dynamic" && !selected,
            } }, additionalFieldDisplay === "dynamic" && (h("div", { key: '430d6b7ec062db38e94360206b1b81ec645d9142', class: "branch-corner" })), h("div", { key: 'd657669620a1a1047f7ce7c524ad74863cdfedb3' }, additionalFieldDisplay === "dynamic" && (h("ic-typography", { key: '89583fec44af079b14161ab9f11eb06337890b0a', variant: "caption" }, h("p", { key: 'b1e27f252e6f43faf95b9f5cd8da1bf792310331', class: "dynamic-text" }, dynamicText))), h("div", { key: 'c5bdfc982b7dbbc1da3dfff5e8d8428df433af50', class: {
                "additional-field-wrapper": additionalFieldDisplay === "static",
            } }, h("slot", { key: 'a1ae20ee1eaf80bd159215a46ddc8b6d4fec7653', name: ADDITIONAL_FIELD })))))));
    }
    static get is() { return "ic-radio-option"; }
    static get originalStyleUrls() {
        return {
            "$": ["ic-radio-option.css"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["ic-radio-option.css"]
        };
    }
    static get properties() {
        return {
            "additionalFieldDisplay": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcAdditionalFieldTypes",
                    "resolved": "\"dynamic\" | \"static\" | undefined",
                    "references": {
                        "IcAdditionalFieldTypes": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcAdditionalFieldTypes"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The style of additionalField that will be displayed if used."
                },
                "getter": false,
                "setter": false,
                "attribute": "additional-field-display",
                "reflect": true,
                "defaultValue": "\"static\""
            },
            "disabled": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the disabled state will be set."
                },
                "getter": false,
                "setter": false,
                "attribute": "disabled",
                "reflect": false,
                "defaultValue": "false"
            },
            "dynamicText": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The text to be displayed when dynamic."
                },
                "getter": false,
                "setter": false,
                "attribute": "dynamic-text",
                "reflect": false,
                "defaultValue": "\"This selection requires additional answers\""
            },
            "form": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The <form> element to associate the radio with."
                },
                "getter": false,
                "setter": false,
                "attribute": "form",
                "reflect": false
            },
            "groupLabel": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The group label for the radio option."
                },
                "getter": false,
                "setter": false,
                "attribute": "group-label",
                "reflect": false
            },
            "label": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The label for the radio option."
                },
                "getter": false,
                "setter": false,
                "attribute": "label",
                "reflect": false
            },
            "name": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "The name for the radio option."
                },
                "getter": false,
                "setter": false,
                "attribute": "name",
                "reflect": false
            },
            "theme": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "IcThemeMode",
                    "resolved": "\"dark\" | \"inherit\" | \"light\" | undefined",
                    "references": {
                        "IcThemeMode": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcThemeMode"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Sets the theme color to the dark or light theme color. \"inherit\" will set the color based on the system settings or ic-theme component."
                },
                "getter": false,
                "setter": false,
                "attribute": "theme",
                "reflect": false,
                "defaultValue": "\"inherit\""
            },
            "value": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": true,
                "optional": false,
                "docs": {
                    "tags": [],
                    "text": "The value for the radio option."
                },
                "getter": false,
                "setter": false,
                "attribute": "value",
                "reflect": false
            },
            "selected": {
                "type": "boolean",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean | undefined",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "If `true`, the radio option will be displayed in a selected state."
                },
                "getter": false,
                "setter": false,
                "attribute": "selected",
                "reflect": true,
                "defaultValue": "false"
            }
        };
    }
    static get states() {
        return {
            "initiallySelected": {}
        };
    }
    static get events() {
        return [{
                "method": "icCheck",
                "name": "icCheck",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Emitted when the radio option is selected."
                },
                "complexType": {
                    "original": "IcValueEventDetail",
                    "resolved": "IcValueEventDetail",
                    "references": {
                        "IcValueEventDetail": {
                            "location": "import",
                            "path": "../../utils/types",
                            "id": "src/utils/types.ts::IcValueEventDetail"
                        }
                    }
                }
            }, {
                "method": "icSelectedChange",
                "name": "icSelectedChange",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Emitted when the radio option is selected or deselected."
                },
                "complexType": {
                    "original": "void",
                    "resolved": "void",
                    "references": {}
                }
            }];
    }
    static get methods() {
        return {
            "setFocus": {
                "complexType": {
                    "signature": "() => Promise<void>",
                    "parameters": [],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "Sets focus on the radio option.",
                    "tags": []
                }
            },
            "setTabIndex": {
                "complexType": {
                    "signature": "(value: number) => Promise<void>",
                    "parameters": [{
                            "name": "value",
                            "type": "number",
                            "docs": ""
                        }],
                    "references": {
                        "Promise": {
                            "location": "global",
                            "id": "global::Promise"
                        }
                    },
                    "return": "Promise<void>"
                },
                "docs": {
                    "text": "",
                    "tags": [{
                            "name": "internal",
                            "text": "Sets the tabIndex of the radio option."
                        }]
                }
            }
        };
    }
    static get elementRef() { return "el"; }
    static get watchers() {
        return [{
                "propName": "disabled",
                "methodName": "watchDisabledHandler"
            }, {
                "propName": "selected",
                "methodName": "watchSelectedHandler"
            }];
    }
    static get listeners() {
        return [{
                "name": "icCheck",
                "method": "handleCheck",
                "target": undefined,
                "capture": false,
                "passive": false
            }, {
                "name": "icChange",
                "method": "additionalFieldValueHandler",
                "target": undefined,
                "capture": false,
                "passive": false
            }];
    }
}
//# sourceMappingURL=ic-radio-option.js.map
